=begin
------------------------------------------------------------------------
Copyright 2008, TIG (c)
Permission to use, copy, modify, and distribute this software for any 
purpose, and currently without fee, is hereby granted, provided that  
this text and the above copyright (c) notice appear in all copies.
This software is provided "as is" and without any express or implied 
warranties, including, without limitation, the implied warranties of 
merchantability and fitness for a particular purpose.
------------------------------------------------------------------------
Name:          	Global_Material_Change.rb
Description:   	Changes Material Globally in Selection - 
				including mining into Groups/Definitions.
Author:       	TIG
Usage:			1. Select entities.
				[IF there's no selection ALL active entities are used] 
				2. Activate the script from the Plugins Menu.
				3. Choose Old Material [to be replaced].
				[<< DEFAULT >>=NO Material, << ALL >>=ALL Materials & 
				then Alphabetical list]				
				4. Choose the desired New Material.
				[<< DEFAULT >> = No Material]
				To access Materials that are off the visible list type 
				the first letter - e.g. 'W' and use up/down arrow-keys 
				to move to the one you want e.g. 'W'hite...
History:		1.0 20080522	First issue.
				1.1 20080523	Lists alphabetical & guidance on arrows.
								Progress 'ticker' added...
=end
#-----------------------------------------------------------------------------
require 'sketchup.rb' 
class GlobalMaterial 
  def change(entity,old_mat,new_mat)  
    entity.definition.entities.each{|e|change(e,old_mat,new_mat)}if entity.typename=="ComponentInstance"
    entity.entities.each{|e|change(e,old_mat,new_mat)}if entity.typename=="Group"
    if old_mat=="<< ALL >>"
	  entity.material=new_mat
      entity.back_material=new_mat if entity.typename=="Face"
	elsif old_mat==nil
	  entity.material=new_mat if entity.material==nil
      entity.back_material=new_mat if entity.typename=="Face" && entity.back_material==nil
	else
	  entity.material=new_mat if entity.material==old_mat
      entity.back_material=new_mat if entity.typename=="Face" && entity.back_material==old_mat
	end#if
  end#def change 
  def process()
    # get the old and new materials
	model=Sketchup.active_model
	mats=model.materials
	matNames=["<< DEFAULT >>","<< ALL >>"]
	matNstmp=[]
	mats.each{|m|matNstmp.push(m.display_name)}
	matNstmp.sort!
	matNames=matNames+matNstmp
	matList=matNames.join('|')
	# dialogs
	info=[matList]
    prompts=["OLD Material to Replace: "]
    values=["<< DEFAULT >>"]
    results=inputbox(prompts,values,info,"Global Material Changer")
    return nil if ! results
	omat=results[0]
	if omat=="<< ALL >>"
	  return nil if UI.messagebox("This will affect ALL Materials !\nContinue ?",MB_YESNO,"ALL ?")==7### 6=YES 7=NO
	  old_mat="<< ALL >>"
	elsif omat=="<< DEFAULT >>"
	  old_mat=nil
	else
	  old_mat=""; mats.each{|m|old_mat=m if m.display_name==omat}
	end#if
	#
	matNames=matNames-["<< ALL >>"]
	matList=matNames.join('|')
	info=[matList]
    prompts=["NEW Material to Add: "]
    values=["<< DEFAULT >>"]
    results=inputbox(prompts,values,info,"Global Material Changer")
    return nil if ! results
	nmat=results[0]
	if nmat=="<< DEFAULT >>"
	  new_mat=nil
	else
	  new_mat=""; mats.each{|m|new_mat=m if m.display_name==nmat}
	end#if
	# now work on ALL entities   
	if model.selection[0]
	  oticker=model.selection.to_a.length; ticker=1
	  model.selection.each{|e|change(e,old_mat,new_mat)
	    Sketchup::set_status_text("Global Material Change - Entity Progress = "+ticker.to_s+" of "+oticker.to_s,SB_PROMPT)
		ticker=ticker+1
      }
	else
	  oticker=model.active_entities.to_a.length; ticker=1
	  model.active_entities.each{|e|change(e,old_mat,new_mat)
	    Sketchup::set_status_text("Global Material Change - Entity Progress = "+ticker.to_s+" of "+oticker.to_s,SB_PROMPT)
		ticker=ticker+1
	  }
	end#if
  end#def process 
end#class
#-----------------------------------------------------------------------
def global_material_change
  model=Sketchup.active_model
  model.start_operation("Global Material Change")
    if ! model.selection[0]
      return nil if UI.messagebox("This will affect ALL Active Entities !\nContinue ?",MB_YESNO,"ALL ?")==7### 6=YES 7=NO
    end#if
    GlobalMaterial.new.process
  model.commit_operation
end#def
#-----------------------------------------------------------------------
is_this_file=File.basename(__FILE__)
if not file_loaded?(is_this_file)
  UI.menu("Plugins").add_item("Global Material Changer"){global_material_change}
end#if
#-----------------------------------------------------------------------
file_loaded(is_this_file)
